Package: recountmethylation
Version: 1.22.0
Title: Access and analyze public DNA methylation array data
        compilations
Authors@R: c(person(c("Sean", "K"), "Maden", role = c("cre", "aut"),
        email = "maden@ohsu.edu", comment = c(ORCID = "0000-0002-2212-4894")),
    person("Brian", "Walsh", role = "aut", email = "walsbr@ohsu.edu", comment = c(ORCID = "0000-0003-0913-1528")),
    person("Kyle", "Ellrott", role = "aut", email = "kellrott@gmail.com", comment = c(ORCID = "0000-0002-6573-5900")),
    person("Kasper D", "Hansen", role = "aut", email = "hansen@jhsph.edu", comment = c(ORCID = "0000-0003-0086-0687")),
    person("Reid F", "Thompson", role = "aut", comment = c(ORCID = "0000-0003-3661-5296")),
    person("Abhinav", "Nellore", role = "aut", email = "nellore@ohsu.edu", comment = c(ORCID = "0000-0001-8145-1484")))
Description: Resources for cross-study analyses of public DNAm array
        data from NCBI GEO repo, produced using Illumina's Infinium
        HumanMethylation450K (HM450K) and MethylationEPIC (EPIC)
        platforms. Provided functions enable download, summary, and
        filtering of large compilation files. Vignettes detail
        background about file formats, example analyses, and more. Note
        the disclaimer on package load and consult the main manuscripts
        for further info.
License: Artistic-2.0
Encoding: UTF-8
URL: https://github.com/metamaden/recountmethylation
BugReports: https://github.com/metamaden/recountmethylation/issues
Depends: R (>= 4.1)
Imports: minfi, HDF5Array, rhdf5, S4Vectors, utils, methods, RCurl,
        R.utils, BiocFileCache, basilisk, reticulate,
        DelayedMatrixStats
Suggests: minfiData, minfiDataEPIC, knitr, testthat, ggplot2,
        gridExtra, rmarkdown, BiocStyle, GenomicRanges, limma,
        ExperimentHub, AnnotationHub
VignetteBuilder: knitr
biocViews: DNAMethylation, Epigenetics, Microarray, MethylationArray,
        ExperimentHub
RoxygenNote: 7.2.0
Config/pak/sysreqs: make libbz2-dev libicu-dev liblzma-dev libpng-dev
        libxml2-dev libssl-dev python3 libx11-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:53:34 UTC
RemoteUrl: https://github.com/bioc/recountmethylation
RemoteRef: RELEASE_3_23
RemoteSha: 2cb8c06a8358866b30c5bdf8e89f3aa29e1897f0
NeedsCompilation: no
Packaged: 2026-04-30 06:13:30 UTC; root
Author: Sean K Maden [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-2212-4894>),
  Brian Walsh [aut] (ORCID: <https://orcid.org/0000-0003-0913-1528>),
  Kyle Ellrott [aut] (ORCID: <https://orcid.org/0000-0002-6573-5900>),
  Kasper D Hansen [aut] (ORCID: <https://orcid.org/0000-0003-0086-0687>),
  Reid F Thompson [aut] (ORCID: <https://orcid.org/0000-0003-3661-5296>),
  Abhinav Nellore [aut] (ORCID: <https://orcid.org/0000-0001-8145-1484>)
Maintainer: Sean K Maden <maden@ohsu.edu>
Built: R 4.6.0; ; 2026-04-30 06:43:25 UTC; unix
