# Generated by roxygen2: do not edit by hand
importFrom("methods", "is")
importFrom("utils", "download.file", "packageDescription")
export(bactrl)
export(bathresh)
export(data_mdpost)
export(gds_idat2rg)
export(gds_idatquery)
export(get_crossreactive_cpgs)
export(get_qcsignal)
export(get_rmdl)
export(get_servermatrix)
export(getdb_h5_rg)
export(getdb_h5_test)
export(getdb_h5se_gm)
export(getdb_h5se_gr)
export(getdb_h5se_rg)
export(getdb_h5se_test)
export(getrg)
export(hread)
export(matchds_1to2)
export(rgse)
export(servermatrix)
export(smfilt)
